@@ -333,7 +333,7 @@ pref("mail.showCondensedAddresses", true); // show the friendly display name for
 
 pref("mailnews.attachments.display.start_expanded", false);
 // hidden pref for changing how we present attachments in the message pane
-pref("mail.pane_config.dynamic",            0);
+pref("mail.pane_config.dynamic", 2);
 pref("mailnews.reuse_thread_window2",     true);
 pref("editor.singleLine.pasteNewlines", 4);  // substitute commas for new lines in single line text boxes
 pref("editor.CR_creates_new_p", true);
@@ -1412,9 +1412,14 @@ pref("services.sync.engine.prefs", false);
 #endif
 
 // Donation appeal.
-pref("app.donation.eoy.version", 2);
+pref("app.donation.eoy.version", 5);
 pref("app.donation.eoy.version.viewed", 0);
-pref("app.donation.eoy.url", "https://www.thunderbird.net/thunderbird/115.0/appeal/");
+pref("app.donation.eoy.url", "https://www.thunderbird.net/thunderbird/115.0/spring24/");
+
+// Survey
+pref("app.survey.version", 1);
+pref("app.survey.version.viewed", 0);
+pref("app.survey.url", "https://www.thunderbird.net/survey/?utm_campaign=survey-May_24");
 
 // IMAP-JS disabled, Bug 1707547.
 pref("mailnews.imap.jsmodule", false);
