@@ -24,6 +24,7 @@
 #include "mozIExtensionProcessScript.h"
 #include "nsEscape.h"
 #include "nsGkAtoms.h"
+#include "nsHashKeys.h"
 #include "nsIChannel.h"
 #include "nsIContentPolicy.h"
 #include "mozilla/dom/Document.h"
@@ -47,7 +48,16 @@ using dom::Promise;
 
 #define DEFAULT_CSP_PREF \
   "extensions.webextensions.default-content-security-policy"
-#define DEFAULT_DEFAULT_CSP "script-src 'self'; object-src 'self';"
+#define DEFAULT_DEFAULT_CSP "script-src 'self' 'wasm-unsafe-eval';"
+
+#define DEFAULT_CSP_PREF_V3 \
+  "extensions.webextensions.default-content-security-policy.v3"
+#define DEFAULT_DEFAULT_CSP_V3 "script-src 'self'; upgrade-insecure-requests;"
+
+#define RESTRICTED_DOMAINS_PREF "extensions.webextensions.restrictedDomains"
+
+#define QUARANTINED_DOMAINS_PREF "extensions.quarantinedDomains.list"
+#define QUARANTINED_DOMAINS_ENABLED "extensions.quarantinedDomains.enabled"
 
 #define OBS_TOPIC_PRELOAD_SCRIPT "web-extension-preload-content-script"
 #define OBS_TOPIC_LOAD_SCRIPT "web-extension-load-content-script"
@@ -58,6 +68,14 @@ static const char kDocElementInserted[] = "initial-document-element-inserted";
  * ExtensionPolicyService
  *****************************************************************************/
 
+using CoreByHostMap = nsTHashMap<nsCStringASCIICaseInsensitiveHashKey,
+                                 RefPtr<extensions::WebExtensionPolicyCore>>;
+
+static StaticRWLock sEPSLock;
+static StaticAutoPtr<CoreByHostMap> sCoreByHost MOZ_GUARDED_BY(sEPSLock);
+static StaticRefPtr<AtomSet> sRestrictedDomains MOZ_GUARDED_BY(sEPSLock);
+static StaticRefPtr<AtomSet> sQuarantinedDomains MOZ_GUARDED_BY(sEPSLock);
+
 /* static */
 mozIExtensionProcessScript& ExtensionPolicyService::ProcessScript() {
   static nsCOMPtr<mozIExtensionProcessScript> sProcessScript;
@@ -68,13 +86,14 @@ mozIExtensionProcessScript& ExtensionPolicyService::ProcessScript() {
     sProcessScript =
         do_ImportModule("resource://gre/modules/ExtensionProcessScript.jsm",
                         "ExtensionProcessScript");
-    MOZ_RELEASE_ASSERT(sProcessScript);
     ClearOnShutdown(&sProcessScript);
   }
   return *sProcessScript;
 }
 
 /* static */ ExtensionPolicyService& ExtensionPolicyService::GetSingleton() {
+  MOZ_ASSERT(NS_IsMainThread());
+
   static RefPtr<ExtensionPolicyService> sExtensionPolicyService;
 
   if (MOZ_UNLIKELY(!sExtensionPolicyService)) {
@@ -85,17 +104,42 @@ mozIExtensionProcessScript& ExtensionPolicyService::ProcessScript() {
   return *sExtensionPolicyService.get();
 }
 
+/* static */
+RefPtr<extensions::WebExtensionPolicyCore>
+ExtensionPolicyService::GetCoreByHost(const nsACString& aHost) {
+  StaticAutoReadLock lock(sEPSLock);
+  return sCoreByHost ? sCoreByHost->Get(aHost) : nullptr;
+}
+
 ExtensionPolicyService::ExtensionPolicyService() {
   mObs = services::GetObserverService();
   MOZ_RELEASE_ASSERT(mObs);
 
   mDefaultCSP.SetIsVoid(true);
+  mDefaultCSPV3.SetIsVoid(true);
 
   RegisterObservers();
+
+  {
+    StaticAutoWriteLock lock(sEPSLock);
+    MOZ_DIAGNOSTIC_ASSERT(!sCoreByHost,
+                          "ExtensionPolicyService created twice?");
+    sCoreByHost = new CoreByHostMap();
+  }
+
+  UpdateRestrictedDomains();
+  UpdateQuarantinedDomains();
 }
 
 ExtensionPolicyService::~ExtensionPolicyService() {
   UnregisterWeakMemoryReporter(this);
+
+  {
+    StaticAutoWriteLock lock(sEPSLock);
+    sCoreByHost = nullptr;
+    sRestrictedDomains = nullptr;
+    sQuarantinedDomains = nullptr;
+  }
 }
 
 bool ExtensionPolicyService::UseRemoteExtensions() const {
@@ -116,6 +160,10 @@ bool ExtensionPolicyService::IsExtensionProcess() const {
   return !isRemote && XRE_IsParentProcess();
 }
 
+bool ExtensionPolicyService::GetQuarantinedDomainsEnabled() const {
+  return Preferences::GetBool(QUARANTINED_DOMAINS_ENABLED);
+}
+
 WebExtensionPolicy* ExtensionPolicyService::GetByURL(const URLInfo& aURL) {
   if (aURL.Scheme() == nsGkAtoms::moz_extension) {
     return GetByHost(aURL.Host());
@@ -123,6 +171,13 @@ WebExtensionPolicy* ExtensionPolicyService::GetByURL(const URLInfo& aURL) {
   return nullptr;
 }
 
+WebExtensionPolicy* ExtensionPolicyService::GetByHost(
+    const nsACString& aHost) const {
+  AssertIsOnMainThread();
+  RefPtr<WebExtensionPolicyCore> core = GetCoreByHost(aHost);
+  return core ? core->GetMainThreadPolicy() : nullptr;
+}
+
 void ExtensionPolicyService::GetAll(
     nsTArray<RefPtr<WebExtensionPolicy>>& aResult) {
   AppendToArray(aResult, mExtensions.Values());
@@ -138,8 +193,11 @@ bool ExtensionPolicyService::RegisterExtension(WebExtensionPolicy& aPolicy) {
   }
 
   mExtensions.InsertOrUpdate(aPolicy.Id(), RefPtr{&aPolicy});
-  mExtensionHosts.InsertOrUpdate(aPolicy.MozExtensionHostname(),
-                                 RefPtr{&aPolicy});
+
+  {
+    StaticAutoWriteLock lock(sEPSLock);
+    sCoreByHost->InsertOrUpdate(aPolicy.MozExtensionHostname(), aPolicy.Core());
+  }
   return true;
 }
 
@@ -153,7 +211,11 @@ bool ExtensionPolicyService::UnregisterExtension(WebExtensionPolicy& aPolicy) {
   }
 
   mExtensions.Remove(aPolicy.Id());
-  mExtensionHosts.Remove(aPolicy.MozExtensionHostname());
+
+  {
+    StaticAutoWriteLock lock(sEPSLock);
+    sCoreByHost->Remove(aPolicy.MozExtensionHostname());
+  }
   return true;
 }
 
@@ -214,6 +276,10 @@ void ExtensionPolicyService::RegisterObservers() {
   }
 
   Preferences::AddStrongObserver(this, DEFAULT_CSP_PREF);
+  Preferences::AddStrongObserver(this, DEFAULT_CSP_PREF_V3);
+  Preferences::AddStrongObserver(this, RESTRICTED_DOMAINS_PREF);
+  Preferences::AddStrongObserver(this, QUARANTINED_DOMAINS_PREF);
+  Preferences::AddStrongObserver(this, QUARANTINED_DOMAINS_ENABLED);
 }
 
 void ExtensionPolicyService::UnregisterObservers() {
@@ -224,6 +290,10 @@ void ExtensionPolicyService::UnregisterObservers() {
   }
 
   Preferences::RemoveObserver(this, DEFAULT_CSP_PREF);
+  Preferences::RemoveObserver(this, DEFAULT_CSP_PREF_V3);
+  Preferences::RemoveObserver(this, RESTRICTED_DOMAINS_PREF);
+  Preferences::RemoveObserver(this, QUARANTINED_DOMAINS_PREF);
+  Preferences::RemoveObserver(this, QUARANTINED_DOMAINS_ENABLED);
 }
 
 nsresult ExtensionPolicyService::Observe(nsISupports* aSubject,
@@ -245,6 +315,13 @@ nsresult ExtensionPolicyService::Observe(nsISupports* aSubject,
     const char* pref = converted.get();
     if (!strcmp(pref, DEFAULT_CSP_PREF)) {
       mDefaultCSP.SetIsVoid(true);
+    } else if (!strcmp(pref, DEFAULT_CSP_PREF_V3)) {
+      mDefaultCSPV3.SetIsVoid(true);
+    } else if (!strcmp(pref, RESTRICTED_DOMAINS_PREF)) {
+      UpdateRestrictedDomains();
+    } else if (!strcmp(pref, QUARANTINED_DOMAINS_PREF) ||
+               !strcmp(pref, QUARANTINED_DOMAINS_ENABLED)) {
+      UpdateQuarantinedDomains();
     }
   }
   return NS_OK;
@@ -348,8 +425,8 @@ nsresult ExtensionPolicyService::InjectContentScripts(
     MOZ_TRY(ExecuteContentScripts(jsapi.cx(), inner,
                                   GetScripts(RunAt::Document_start))
                 ->ThenWithCycleCollectedArgs(
-                    [](JSContext* aCx, JS::HandleValue aValue,
-                       ExtensionPolicyService* aSelf,
+                    [](JSContext* aCx, JS::Handle<JS::Value> aValue,
+                       ErrorResult& aRv, ExtensionPolicyService* aSelf,
                        nsPIDOMWindowInner* aInner, Scripts&& aScripts) {
                       return aSelf->ExecuteContentScripts(aCx, aInner, aScripts)
                           .forget();
@@ -357,8 +434,8 @@ nsresult ExtensionPolicyService::InjectContentScripts(
                     this, inner, GetScripts(RunAt::Document_end))
                 .andThen([&](auto aPromise) {
                   return aPromise->ThenWithCycleCollectedArgs(
-                      [](JSContext* aCx, JS::HandleValue aValue,
-                         ExtensionPolicyService* aSelf,
+                      [](JSContext* aCx, JS::Handle<JS::Value> aValue,
+                         ErrorResult& aRv, ExtensionPolicyService* aSelf,
                          nsPIDOMWindowInner* aInner, Scripts&& aScripts) {
                         return aSelf
                             ->ExecuteContentScripts(aCx, aInner, aScripts)
@@ -505,6 +582,55 @@ void ExtensionPolicyService::CheckContentScripts(const DocInfo& aDocInfo,
   }
 }
 
+/* static */
+RefPtr<AtomSet> ExtensionPolicyService::RestrictedDomains() {
+  StaticAutoReadLock lock(sEPSLock);
+  return sRestrictedDomains;
+}
+
+/* static */
+RefPtr<AtomSet> ExtensionPolicyService::QuarantinedDomains() {
+  StaticAutoReadLock lock(sEPSLock);
+  return sQuarantinedDomains;
+}
+
+void ExtensionPolicyService::UpdateRestrictedDomains() {
+  nsAutoCString eltsString;
+  Unused << Preferences::GetCString(RESTRICTED_DOMAINS_PREF, eltsString);
+
+  AutoTArray<nsString, 32> elts;
+  for (const nsACString& elt : eltsString.Split(',')) {
+    elts.AppendElement(NS_ConvertUTF8toUTF16(elt));
+    elts.LastElement().StripWhitespace();
+  }
+  RefPtr<AtomSet> atomSet = new AtomSet(elts);
+
+  StaticAutoWriteLock lock(sEPSLock);
+  sRestrictedDomains = atomSet;
+}
+
+void ExtensionPolicyService::UpdateQuarantinedDomains() {
+  if (!GetQuarantinedDomainsEnabled()) {
+    StaticAutoWriteLock lock(sEPSLock);
+    sQuarantinedDomains = nullptr;
+    return;
+  }
+
+  nsAutoCString eltsString;
+  AutoTArray<nsString, 32> elts;
+  if (NS_SUCCEEDED(
+          Preferences::GetCString(QUARANTINED_DOMAINS_PREF, eltsString))) {
+    for (const nsACString& elt : eltsString.Split(',')) {
+      elts.AppendElement(NS_ConvertUTF8toUTF16(elt));
+      elts.LastElement().StripWhitespace();
+    }
+  }
+  RefPtr<AtomSet> atomSet = new AtomSet(elts);
+
+  StaticAutoWriteLock lock(sEPSLock);
+  sQuarantinedDomains = atomSet;
+}
+
 /*****************************************************************************
  * nsIAddonPolicyService
  *****************************************************************************/
@@ -522,6 +648,19 @@ nsresult ExtensionPolicyService::GetDefaultCSP(nsAString& aDefaultCSP) {
   return NS_OK;
 }
 
+nsresult ExtensionPolicyService::GetDefaultCSPV3(nsAString& aDefaultCSP) {
+  if (mDefaultCSPV3.IsVoid()) {
+    nsresult rv = Preferences::GetString(DEFAULT_CSP_PREF_V3, mDefaultCSPV3);
+    if (NS_FAILED(rv)) {
+      mDefaultCSPV3.AssignLiteral(DEFAULT_DEFAULT_CSP_V3);
+    }
+    mDefaultCSPV3.SetIsVoid(false);
+  }
+
+  aDefaultCSP.Assign(mDefaultCSPV3);
+  return NS_OK;
+}
+
 nsresult ExtensionPolicyService::GetBaseCSP(const nsAString& aAddonId,
                                             nsAString& aResult) {
   if (WebExtensionPolicy* policy = GetByID(aAddonId)) {
@@ -606,8 +745,7 @@ nsresult ExtensionPolicyService::ExtensionURIToAddonId(nsIURI* aURI,
   return NS_OK;
 }
 
-NS_IMPL_CYCLE_COLLECTION(ExtensionPolicyService, mExtensions, mExtensionHosts,
-                         mObservers)
+NS_IMPL_CYCLE_COLLECTION(ExtensionPolicyService, mExtensions, mObservers)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(ExtensionPolicyService)
   NS_INTERFACE_MAP_ENTRY(nsIAddonPolicyService)
