@@ -9,7 +9,7 @@
 #include "Common.h"
 #include "mozilla/Components.h"
 #include "mozilla/ResultVariant.h"
-#include "mozilla/Tuple.h"
+
 #include "mozilla/dom/ToJSValue.h"
 #include "mozilla/glean/bindings/HistogramGIFFTMap.h"
 #include "mozilla/glean/fog_ffi_generated.h"
@@ -25,7 +25,7 @@ extern "C" NS_EXPORT void GIFFT_TimingDistributionStart(
   auto mirrorId = mozilla::glean::HistogramIdForMetric(aMetricId);
   if (mirrorId) {
     mozilla::glean::GetTimerIdToStartsLock().apply([&](auto& lock) {
-      auto tuple = mozilla::MakeTuple(aMetricId, aTimerId);
+      auto tuple = mozilla::glean::MetricTimerTuple{aMetricId, aTimerId};
       // It should be all but impossible for anyone to have already inserted
       // this timer for this metric given the monotonicity of timer ids.
       (void)NS_WARN_IF(lock.ref()->Remove(tuple));
@@ -40,8 +40,8 @@ extern "C" NS_EXPORT void GIFFT_TimingDistributionStopAndAccumulate(
   auto mirrorId = mozilla::glean::HistogramIdForMetric(aMetricId);
   if (mirrorId) {
     mozilla::glean::GetTimerIdToStartsLock().apply([&](auto& lock) {
-      auto optStart =
-          lock.ref()->Extract(mozilla::MakeTuple(aMetricId, aTimerId));
+      auto tuple = mozilla::glean::MetricTimerTuple{aMetricId, aTimerId};
+      auto optStart = lock.ref()->Extract(tuple);
       // The timer might not be in the map to be removed if it's already been
       // cancelled or stop_and_accumulate'd.
       if (!NS_WARN_IF(!optStart)) {
@@ -68,8 +68,8 @@ extern "C" NS_EXPORT void GIFFT_TimingDistributionCancel(
     mozilla::glean::GetTimerIdToStartsLock().apply([&](auto& lock) {
       // The timer might not be in the map to be removed if it's already been
       // cancelled or stop_and_accumulate'd.
-      (void)NS_WARN_IF(
-          !lock.ref()->Remove(mozilla::MakeTuple(aMetricId, aTimerId)));
+      auto tuple = mozilla::glean::MetricTimerTuple{aMetricId, aTimerId};
+      (void)NS_WARN_IF(!lock.ref()->Remove(tuple));
     });
   }
 }
@@ -101,7 +101,7 @@ void TimingDistributionMetric::Cancel(const TimerId&& aId) const {
 Result<Maybe<DistributionData>, nsCString>
 TimingDistributionMetric::TestGetValue(const nsACString& aPingName) const {
   nsCString err;
-  if (fog_timing_distribution_test_get_error(mId, &aPingName, &err)) {
+  if (fog_timing_distribution_test_get_error(mId, &err)) {
     return Err(err);
   }
   if (!fog_timing_distribution_test_has_value(mId, &aPingName)) {
@@ -121,7 +121,8 @@ NS_IMPL_CLASSINFO(GleanTimingDistribution, nullptr, 0, {0})
 NS_IMPL_ISUPPORTS_CI(GleanTimingDistribution, nsIGleanTimingDistribution)
 
 NS_IMETHODIMP
-GleanTimingDistribution::Start(JSContext* aCx, JS::MutableHandleValue aResult) {
+GleanTimingDistribution::Start(JSContext* aCx,
+                               JS::MutableHandle<JS::Value> aResult) {
   if (!dom::ToJSValue(aCx, mTimingDist.Start(), aResult)) {
     return NS_ERROR_FAILURE;
   }
@@ -143,7 +144,7 @@ GleanTimingDistribution::Cancel(uint64_t aId) {
 NS_IMETHODIMP
 GleanTimingDistribution::TestGetValue(const nsACString& aPingName,
                                       JSContext* aCx,
-                                      JS::MutableHandleValue aResult) {
+                                      JS::MutableHandle<JS::Value> aResult) {
   auto result = mTimingDist.TestGetValue(aPingName);
   if (result.isErr()) {
     aResult.set(JS::UndefinedValue());
@@ -157,7 +158,7 @@ GleanTimingDistribution::TestGetValue(const nsACString& aPingName,
   } else {
     // Build return value of the form: { sum: #, values: {bucket1: count1,
     // ...}
-    JS::RootedObject root(aCx, JS_NewPlainObject(aCx));
+    JS::Rooted<JSObject*> root(aCx, JS_NewPlainObject(aCx));
     if (!root) {
       return NS_ERROR_FAILURE;
     }
@@ -166,7 +167,7 @@ GleanTimingDistribution::TestGetValue(const nsACString& aPingName,
                            JSPROP_ENUMERATE)) {
       return NS_ERROR_FAILURE;
     }
-    JS::RootedObject valuesObj(aCx, JS_NewPlainObject(aCx));
+    JS::Rooted<JSObject*> valuesObj(aCx, JS_NewPlainObject(aCx));
     if (!valuesObj ||
         !JS_DefineProperty(aCx, root, "values", valuesObj, JSPROP_ENUMERATE)) {
       return NS_ERROR_FAILURE;
