@@ -4,17 +4,16 @@
 // Loaded into the same scope as head_xpc.js
 /* import-globals-from head_xpc.js */
 
-const { Preferences } = ChromeUtils.import(
-  "resource://gre/modules/Preferences.jsm"
+const { Preferences } = ChromeUtils.importESModule(
+  "resource://gre/modules/Preferences.sys.mjs"
 );
 const { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
 
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
-const { NormandyApi } = ChromeUtils.import(
-  "resource://normandy/lib/NormandyApi.jsm"
+const { NormandyApi } = ChromeUtils.importESModule(
+  "resource://normandy/lib/NormandyApi.sys.mjs"
 );
-const { NormandyTestUtils } = ChromeUtils.import(
-  "resource://testing-common/NormandyTestUtils.jsm"
+const { NormandyTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/NormandyTestUtils.sys.mjs"
 );
 
 const CryptoHash = Components.Constructor(
@@ -43,7 +42,7 @@ class MockResponse {
 }
 
 function withServer(server) {
-  return function(testFunction) {
+  return function (testFunction) {
     return NormandyTestUtils.decorate(
       NormandyTestUtils.withMockPreferences(),
       async function inner({ mockPreferences, ...args }) {
@@ -77,7 +76,7 @@ function makeMockApiServer(directory) {
   const server = new HttpServer();
   server.registerDirectory("/", directory);
 
-  server.setIndexHandler(async function(request, response) {
+  server.setIndexHandler(async function (request, response) {
     response.processAsync();
     const dir = request.getProperty("directory");
     const index = dir.clone();
@@ -91,7 +90,7 @@ function makeMockApiServer(directory) {
     }
 
     try {
-      const contents = await OS.File.read(index.path, { encoding: "utf-8" });
+      const contents = await IOUtils.readUTF8(index.path);
       response.write(contents);
     } catch (e) {
       response.setStatusLine("1.1", 500, "Server error");
