@@ -21,9 +21,7 @@ add_task(async function test_initialize() {
     Services.prefs.clearUserPref("signon.autofillForms");
     Services.prefs.clearUserPref("signon.schemeUpgrades");
   });
-  for (let login of loginList()) {
-    Services.logins.addLogin(login);
-  }
+  await Services.logins.addLogins(loginList());
 });
 
 /**
@@ -35,7 +33,7 @@ add_task(async function test_context_menu_iframe_fill() {
       gBrowser,
       url: TEST_ORIGIN + IFRAME_PAGE_PATH,
     },
-    async function(browser) {
+    async function (browser) {
       await openPasswordContextMenu(
         browser,
         "#form-basic-password",
@@ -51,7 +49,7 @@ add_task(async function test_context_menu_iframe_fill() {
         return SpecialPowers.spawn(
           browser.browsingContext.children[0],
           [name],
-          function(inputname) {
+          function (inputname) {
             return content.document.getElementById(inputname).value;
           }
         );
@@ -61,14 +59,13 @@ add_task(async function test_context_menu_iframe_fill() {
       );
 
       // Execute the command of the first login menuitem found at the context menu.
-      let firstLoginItem = popupMenu.getElementsByClassName(
-        "context-login-item"
-      )[0];
-      ok(firstLoginItem, "Found the first login item");
+      let firstLoginItem =
+        popupMenu.getElementsByClassName("context-login-item")[0];
+      Assert.ok(firstLoginItem, "Found the first login item");
 
       await TestUtils.waitForTick();
 
-      ok(
+      Assert.ok(
         BrowserTestUtils.is_visible(firstLoginItem),
         "First login menuitem is visible"
       );
@@ -84,12 +81,16 @@ add_task(async function test_context_menu_iframe_fill() {
 
       // Find the used login by it's username.
       let login = getLoginFromUsername(firstLoginItem.label);
-      is(login.password, passwordValue, "Password filled and correct.");
+      Assert.equal(
+        login.password,
+        passwordValue,
+        "Password filled and correct."
+      );
 
       let usernameNewValue = await promiseFrameInputValue(
         "form-basic-username"
       );
-      is(
+      Assert.equal(
         usernameOriginalValue,
         usernameNewValue,
         "Username value was not changed."
@@ -113,7 +114,7 @@ add_task(async function test_context_menu_iframe_sandbox() {
       gBrowser,
       url: TEST_ORIGIN + IFRAME_PAGE_PATH,
     },
-    async function(browser) {
+    async function (browser) {
       info("Opening context menu for test_context_menu_iframe_sandbox");
       await openPasswordContextMenu(
         browser,
@@ -121,7 +122,7 @@ add_task(async function test_context_menu_iframe_sandbox() {
         function checkDisabled() {
           info("checkDisabled for test_context_menu_iframe_sandbox");
           let popupHeader = document.getElementById("fill-login");
-          ok(
+          Assert.ok(
             popupHeader.hidden,
             "Check that the Fill Login menu item is hidden"
           );
@@ -144,17 +145,17 @@ add_task(async function test_context_menu_iframe_sandbox_same_origin() {
       gBrowser,
       url: TEST_ORIGIN + IFRAME_PAGE_PATH,
     },
-    async function(browser) {
+    async function (browser) {
       await openPasswordContextMenu(
         browser,
         "#form-basic-password",
         function checkDisabled() {
           let popupHeader = document.getElementById("fill-login");
-          ok(
+          Assert.ok(
             !popupHeader.hidden,
             "Check that the Fill Login menu item is visible"
           );
-          ok(
+          Assert.ok(
             !popupHeader.disabled,
             "Check that the Fill Login menu item is disabled"
           );
