@@ -6,17 +6,15 @@
 
 "use strict";
 
-const { ClientID } = ChromeUtils.import("resource://gre/modules/ClientID.jsm");
-const { TelemetryArchive } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryArchive.jsm"
+const { ClientID } = ChromeUtils.importESModule(
+  "resource://gre/modules/ClientID.sys.mjs"
+);
+const { TelemetryArchive } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryArchive.sys.mjs"
 );
 
-XPCOMUtils.defineLazyGetter(this, "gPingsArchivePath", function() {
-  return OS.Path.join(
-    OS.Constants.Path.profileDir,
-    "datareporting",
-    "archived"
-  );
+XPCOMUtils.defineLazyGetter(this, "gPingsArchivePath", function () {
+  return PathUtils.join(PathUtils.profileDir, "datareporting", "archived");
 });
 
 /**
@@ -24,8 +22,8 @@ XPCOMUtils.defineLazyGetter(this, "gPingsArchivePath", function() {
  * @param {Integer} aArchiveQuota The new quota, in bytes.
  */
 function fakeStorageQuota(aArchiveQuota) {
-  let { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetryStorage.jsm"
+  let { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryStorage.sys.mjs"
   );
   Policy.getArchiveQuota = () => aArchiveQuota;
 }
@@ -33,34 +31,37 @@ function fakeStorageQuota(aArchiveQuota) {
 /**
  * Lists all the valid archived pings and their metadata, sorted by creation date.
  *
- * @param aFileName {String} The filename.
  * @return {Object[]} A list of objects with the extracted data in the form:
  *                  { timestamp: <number>,
  *                    id: <string>,
  *                    type: <string>,
  *                    size: <integer> }
  */
-var getArchivedPingsInfo = async function() {
-  let dirIterator = new OS.File.DirectoryIterator(gPingsArchivePath);
-  let subdirs = (await dirIterator.nextBatch()).filter(e => e.isDir);
+var getArchivedPingsInfo = async function () {
   let archivedPings = [];
 
   // Iterate through the subdirs of |gPingsArchivePath|.
-  for (let dir of subdirs) {
-    let fileIterator = new OS.File.DirectoryIterator(dir.path);
-    let files = (await fileIterator.nextBatch()).filter(e => !e.isDir);
+  for (const dir of await IOUtils.getChildren(gPingsArchivePath)) {
+    const { type } = await IOUtils.stat(dir);
+    if (type != "directory") {
+      continue;
+    }
 
     // Then get a list of the files for the current subdir.
-    for (let f of files) {
+    for (const filePath of await IOUtils.getChildren(dir)) {
+      const fileInfo = await IOUtils.stat(filePath);
+      if (fileInfo.type == "directory") {
+        continue;
+      }
       let pingInfo = TelemetryStorage._testGetArchivedPingDataFromFileName(
-        f.name
+        PathUtils.filename(filePath)
       );
       if (!pingInfo) {
         // This is not a valid archived ping, skip it.
         continue;
       }
       // Find the size of the ping and then add the info to the array.
-      pingInfo.size = (await OS.File.stat(f.path)).size;
+      pingInfo.size = fileInfo.size;
       archivedPings.push(pingInfo);
     }
   }
@@ -117,7 +118,7 @@ add_task(async function test_archivedPings() {
   }
 
   // Check loading the archived pings.
-  let checkLoadingPings = async function() {
+  let checkLoadingPings = async function () {
     for (let data of PINGS) {
       let ping = await TelemetryArchive.promiseArchivedPingById(data.id);
       Assert.equal(ping.id, data.id, "Archived ping should have matching id");
@@ -148,20 +149,20 @@ add_task(async function test_archivedPings() {
   await checkLoadingPings();
 
   // Write invalid pings into the archive with both valid and invalid names.
-  let writeToArchivedDir = async function(
+  let writeToArchivedDir = async function (
     dirname,
     filename,
     content,
     compressed
   ) {
-    const dirPath = OS.Path.join(gPingsArchivePath, dirname);
-    await OS.File.makeDir(dirPath, { ignoreExisting: true });
-    const filePath = OS.Path.join(dirPath, filename);
-    const options = { tmpPath: filePath + ".tmp", noOverwrite: false };
+    const dirPath = PathUtils.join(gPingsArchivePath, dirname);
+    await IOUtils.makeDirectory(dirPath, { ignoreExisting: true });
+    const filePath = PathUtils.join(dirPath, filename);
+    const options = { tmpPath: filePath + ".tmp", mode: "overwrite" };
     if (compressed) {
-      options.compression = "lz4";
+      options.compress = true;
     }
-    await OS.File.writeAtomic(filePath, content, options);
+    await IOUtils.writeUTF8(filePath, content, options);
   };
 
   const FAKE_ID1 = "10000000-0123-0123-0123-0123456789a1";
@@ -242,7 +243,7 @@ add_task(async function test_archiveCleanup() {
   const PING_TYPE = "foo";
 
   // Empty the archive.
-  await OS.File.removeDir(gPingsArchivePath);
+  await IOUtils.remove(gPingsArchivePath, { recursive: true });
 
   Telemetry.getHistogramById("TELEMETRY_ARCHIVE_SCAN_PING_COUNT").clear();
   Telemetry.getHistogramById("TELEMETRY_ARCHIVE_DIRECTORIES_COUNT").clear();
@@ -278,7 +279,7 @@ add_task(async function test_archiveCleanup() {
   let expectedPrunedInfo = [];
   let expectedNotPrunedInfo = [];
 
-  let checkArchive = async function() {
+  let checkArchive = async function () {
     // Check that the pruned pings are not on disk anymore.
     for (let prunedInfo of expectedPrunedInfo) {
       await Assert.rejects(
@@ -292,7 +293,7 @@ add_task(async function test_archiveCleanup() {
         PING_TYPE
       );
       Assert.ok(
-        !(await OS.File.exists(pingPath)),
+        !(await IOUtils.exists(pingPath)),
         "The ping should not be on the disk anymore."
       );
     }
@@ -519,7 +520,7 @@ add_task(async function test_archiveCleanup() {
       PING_TYPE
     ) + "lz4";
   const archivedPingSizeMB = Math.floor(
-    (await OS.File.stat(oversizedPingPath)).size / 1024 / 1024
+    (await IOUtils.stat(oversizedPingPath)).size / 1024 / 1024
   );
 
   // We expect the oversized ping to be pruned when scanning the archive.
